@@ -1,7 +1,8 @@
+use crate::iter::util::Compress;
 #[cfg(not(feature = "on-disk-utxo"))]
 use crate::iter::util::VecMap;
-use crate::iter::util::{Compress, DBCopy};
 use crate::parser::proto::connected_proto::{BlockConnectable, TxConnectable};
+use crate::BitcoinDB;
 #[cfg(feature = "on-disk-utxo")]
 use bitcoin::consensus::{Decodable, Encodable};
 use bitcoin::Block;
@@ -9,15 +10,15 @@ use bitcoin::Block;
 use bitcoin::TxOut;
 #[cfg(not(feature = "on-disk-utxo"))]
 use hash_hasher::HashedMap;
-#[cfg(not(feature = "on-disk-utxo"))]
-use log::warn;
 use log::error;
+#[cfg(not(feature = "on-disk-utxo"))]
+#[cfg(debug_assertions)]
+use log::warn;
 #[cfg(feature = "on-disk-utxo")]
 use rocksdb::WriteOptions;
 #[cfg(feature = "on-disk-utxo")]
 use rocksdb::{WriteBatch, DB};
-use std::sync::atomic::{AtomicBool, Ordering};
-use std::sync::mpsc::Sender;
+use std::sync::mpsc::SyncSender;
 use std::sync::Arc;
 #[cfg(not(feature = "on-disk-utxo"))]
 use std::sync::Mutex;
@@ -31,102 +32,80 @@ pub(crate) fn update_unspent_cache<TBlock>(
     >,
     #[cfg(feature = "on-disk-utxo")] unspent: &Arc<DB>,
     #[cfg(feature = "on-disk-utxo")] write_options: &WriteOptions,
-    db: &DBCopy,
+    db: &BitcoinDB,
     height: u32,
-    error_state: &Arc<AtomicBool>,
-    channel: &Sender<Block>,
+    channel: &SyncSender<Block>,
 ) -> bool
 where
     TBlock: BlockConnectable,
 {
-    // stop new tasks from loading when error
-    if error_state.load(Ordering::SeqCst) {
-        return false;
-    }
+    match db.get_block::<Block>(height as i32) {
+        #[cfg(not(feature = "on-disk-utxo"))]
+        Ok(block) => {
+            let mut new_unspent_cache = Vec::with_capacity(block.txdata.len());
 
-    if let Some(index) = db.block_index.records.get(height as usize) {
-        match db.blk_file.read_block(index.n_file, index.n_data_pos) {
-            #[cfg(not(feature = "on-disk-utxo"))]
-            Ok(block) => {
-                let mut new_unspent_cache = Vec::with_capacity(block.txdata.len());
-
-                // insert new transactions
-                for tx in block.txdata.iter() {
-                    // clone outputs
-                    let txid = tx.txid();
-                    let mut outs: Vec<Option<<TBlock::Tx as TxConnectable>::TOut>> =
-                        Vec::with_capacity(tx.output.len());
-                    for o in tx.output.iter() {
-                        outs.push(Some(o.clone().into()));
-                    }
-
-                    // update unspent cache
-                    let outs: VecMap<<TBlock::Tx as TxConnectable>::TOut> =
-                        VecMap::from_vec(outs.into_boxed_slice());
-                    let new_unspent = Arc::new(Mutex::new(outs));
-                    let txid_compressed = txid.compress();
-
-                    // the new transaction should not be in unspent
-                    #[cfg(debug_assertions)]
-                    if unspent.lock().unwrap().contains_key(&txid_compressed) {
-                        warn!("found duplicate key {}", &txid);
-                    }
-
-                    new_unspent_cache.push((txid_compressed, new_unspent));
+            // insert new transactions
+            for tx in block.txdata.iter() {
+                // clone outputs
+                let txid = tx.txid();
+                let mut outs: Vec<Option<<TBlock::Tx as TxConnectable>::TOut>> =
+                    Vec::with_capacity(tx.output.len());
+                for o in tx.output.iter() {
+                    outs.push(Some(o.clone().into()));
                 }
-                // quick stopping in error state
-                if error_state.load(Ordering::SeqCst) {
-                    return false;
+
+                // update unspent cache
+                let outs: VecMap<<TBlock::Tx as TxConnectable>::TOut> =
+                    VecMap::from_vec(outs.into_boxed_slice());
+                let new_unspent = Arc::new(Mutex::new(outs));
+                let txid_compressed = txid.compress();
+
+                // the new transaction should not be in unspent
+                #[cfg(debug_assertions)]
+                if unspent.lock().unwrap().contains_key(&txid_compressed) {
+                    warn!("found duplicate key {}", &txid);
                 }
-                unspent.lock().unwrap().extend(new_unspent_cache);
-                channel.send(block).unwrap();
-                true
+
+                new_unspent_cache.push((txid_compressed, new_unspent));
             }
+            unspent.lock().unwrap().extend(new_unspent_cache);
+            // if some exception happens in lower stream
+            channel.send(block).is_ok()
+        }
 
-            #[cfg(feature = "on-disk-utxo")]
-            Ok(block) => {
-                let mut batch = WriteBatch::default();
+        #[cfg(feature = "on-disk-utxo")]
+        Ok(block) => {
+            let mut batch = WriteBatch::default();
 
-                // insert new transactions
-                for tx in block.txdata.iter() {
-                    // clone outputs
-                    let txid_compressed = tx.txid().compress();
+            // insert new transactions
+            for tx in block.txdata.iter() {
+                // clone outputs
+                let txid_compressed = tx.txid().compress();
 
-                    let mut n: u32 = 0;
-                    for o in tx.output.iter() {
-                        let key = txo_key(txid_compressed, n);
-                        let value = txo_to_u8(o);
-                        batch.put(key, value);
-                        n += 1;
-                    }
-                }
-                // quick stopping in error state
-                if error_state.load(Ordering::SeqCst) {
-                    return false;
-                }
-                match unspent.write_opt(batch, write_options) {
-                    Ok(_) => {
-                        channel.send(block).unwrap();
-                        true
-                    }
-                    Err(e) => {
-                        error!("failed to write UTXO to cache, error: {}", e);
-                        mutate_result_error(error_state);
-                        false
-                    }
+                let mut n: u32 = 0;
+                for o in tx.output.iter() {
+                    let key = txo_key(txid_compressed, n);
+                    let value = txo_to_u8(o);
+                    batch.put(key, value);
+                    n += 1;
                 }
             }
-
-            Err(_) => {
-                // set error_state to true
-                mutate_result_error(error_state);
-                false
+            match unspent.write_opt(batch, write_options) {
+                Ok(_) => {
+                    // if some exception happens in lower stream
+                    channel.send(block).is_ok()
+                }
+                Err(e) => {
+                    error!("failed to write UTXO to cache, error: {}", e);
+                    false
+                }
             }
         }
-    } else {
-        // set error_state to true
-        mutate_result_error(error_state);
-        false
+
+        Err(_) => {
+            // usually no more block
+            false
+        }
     }
 }
 
@@ -138,18 +117,13 @@ pub(crate) fn connect_outpoints<TBlock>(
         Mutex<HashedMap<u128, Arc<Mutex<VecMap<<TBlock::Tx as TxConnectable>::TOut>>>>>,
     >,
     #[cfg(feature = "on-disk-utxo")] unspent: &Arc<DB>,
-    error_state: &Arc<AtomicBool>,
-    sender: &Sender<TBlock>,
+    sender: &SyncSender<(TBlock, u32)>,
     block: Block,
+    height: u32,
 ) -> bool
 where
     TBlock: BlockConnectable,
 {
-    // stop new tasks from loading when error
-    if error_state.load(Ordering::SeqCst) {
-        return false;
-    }
-
     let block_hash = block.header.block_hash();
     let mut output_block = TBlock::from(block.header, block_hash);
 
@@ -183,7 +157,6 @@ where
             Ok(_) => {}
             Err(e) => {
                 error!("failed to remove key {:?}, error: {}", &key, e);
-                mutate_result_error(error_state);
                 return false;
             }
         }
@@ -244,12 +217,10 @@ where
                     output_tx.add_input(out);
                 } else {
                     error!("cannot find previous outpoint, bad data");
-                    mutate_result_error(error_state);
                     return false;
                 }
             } else {
                 error!("cannot find previous transactions, bad data");
-                mutate_result_error(error_state);
                 return false;
             }
 
@@ -259,25 +230,16 @@ where
                 pos += 1;
             } else {
                 error!("cannot find previous outpoint, bad data");
-                mutate_result_error(error_state);
                 return false;
             }
         }
         output_block.add_tx(output_tx);
     }
 
-    if error_state.load(Ordering::SeqCst) {
-        return false;
-    }
-    sender.send(output_block).unwrap();
+    sender.send((output_block, height)).unwrap();
     true
 }
 
-#[inline]
-fn mutate_result_error(error_state: &Arc<AtomicBool>) {
-    error_state.fetch_or(true, Ordering::SeqCst);
-}
-
 #[inline(always)]
 #[cfg(feature = "on-disk-utxo")]
 fn txo_key(txid_compressed: u128, n: u32) -> Vec<u8> {
